@@ -11,11 +11,12 @@ use std::{
 use anyhow::{anyhow, ensure, Result};
 use bch_bindgen::{
     bcachefs::{self, bch_key, bch_sb_handle},
-    c::{bch2_chacha_encrypt_key, bch_sb_field_crypt},
+    c::{bch2_chacha_encrypt_key, bch_encrypted_key, bch_sb_field_crypt},
     keyutils::{self, keyctl_search},
 };
 use byteorder::{LittleEndian, ReadBytesExt};
 use log::info;
+use rustix::termios;
 use uuid::Uuid;
 use zeroize::{ZeroizeOnDrop, Zeroizing};
 
@@ -25,9 +26,15 @@ const BCH_KEY_MAGIC: &str = "bch**key";
 
 #[derive(Clone, Debug, clap::ValueEnum, strum::Display)]
 pub enum UnlockPolicy {
+    /// Don't ask for passphrase, if the key cannot be found in the keyring just
+    /// fail
     Fail,
+    /// Wait for passphrase to become available before mounting
     Wait,
+    /// Interactively prompt the user for a passphrase
     Ask,
+    /// Try to read the passphrase from `stdin` without prompting
+    Stdin,
 }
 
 impl UnlockPolicy {
@@ -37,9 +44,10 @@ impl UnlockPolicy {
         info!("Using filesystem unlock policy '{self}' on {uuid}");
 
         match self {
-            Self::Fail => Err(anyhow!("no passphrase available")),
+            Self::Fail => KeyHandle::new_from_search(&uuid),
             Self::Wait => Ok(KeyHandle::wait_for_unlock(&uuid)?),
             Self::Ask => Passphrase::new_from_prompt().and_then(|p| KeyHandle::new(sb, &p)),
+            Self::Stdin => Passphrase::new_from_stdin().and_then(|p| KeyHandle::new(sb, &p)),
         }
     }
 }
@@ -63,44 +71,24 @@ impl KeyHandle {
     }
 
     pub fn new(sb: &bch_sb_handle, passphrase: &Passphrase) -> Result<Self> {
-        let bch_key_magic = BCH_KEY_MAGIC.as_bytes().read_u64::<LittleEndian>().unwrap();
-
-        let crypt = sb.sb().crypt().unwrap();
-        let crypt_ptr = (crypt as *const bch_sb_field_crypt).cast_mut();
-
-        let mut output: bch_key =
-            unsafe { bcachefs::derive_passphrase(crypt_ptr, passphrase.get().as_ptr()) };
-
-        let mut key = *crypt.key();
-
-        let ret = unsafe {
-            bch2_chacha_encrypt_key(
-                ptr::addr_of_mut!(output),
-                sb.sb().nonce(),
-                ptr::addr_of_mut!(key).cast(),
-                mem::size_of_val(&key),
-            )
-        };
-
-        ensure!(ret == 0, "chacha decryption failure");
-        ensure!(key.magic == bch_key_magic, "failed to verify passphrase");
-
         let key_name = Self::format_key_name(&sb.sb().uuid());
         let key_name = CStr::as_ptr(&key_name);
         let key_type = c_str!("user");
 
+        let (passphrase_key, _sb_key) = passphrase.check(sb)?;
+
         let key_id = unsafe {
             keyutils::add_key(
                 key_type,
                 key_name,
-                ptr::addr_of!(output).cast(),
-                mem::size_of_val(&output),
+                ptr::addr_of!(passphrase_key).cast(),
+                mem::size_of_val(&passphrase_key),
                 keyutils::KEY_SPEC_USER_KEYRING,
             )
         };
 
         if key_id > 0 {
-            info!("Found key in keyring");
+            info!("Added key to keyring");
             Ok(KeyHandle {
                 _uuid: sb.sb().uuid(),
                 _id:   c_long::from(key_id),
@@ -110,7 +98,7 @@ impl KeyHandle {
         }
     }
 
-    fn search_keyring(keyring: i32, key_name: &CStr) -> Result<i64> {
+    fn search_keyring(keyring: i32, key_name: &CStr) -> Result<c_long> {
         let key_name = CStr::as_ptr(key_name);
         let key_type = c_str!("user");
 
@@ -154,26 +142,47 @@ impl Passphrase {
         &self.0
     }
 
-    // blocks indefinitely if no input is available on stdin
-    fn new_from_prompt() -> Result<Self> {
-        let passphrase = if stdin().is_terminal() {
-            Zeroizing::new(rpassword::prompt_password("Enter passphrase: ")?)
+    pub fn new() -> Result<Self> {
+        if stdin().is_terminal() {
+            Self::new_from_prompt()
         } else {
-            info!("Trying to read passphrase from stdin...");
-            let mut line = Zeroizing::new(String::new());
-            stdin().read_line(&mut line)?;
-            line
-        };
-
-        Ok(Self(CString::new(passphrase.trim_end_matches('\n'))?))
+            Self::new_from_stdin()
+        }
     }
 
-    pub fn new_from_file(sb: &bch_sb_handle, passphrase_file: impl AsRef<Path>) -> Result<Self> {
+    // blocks indefinitely if no input is available on stdin
+    pub fn new_from_prompt() -> Result<Self> {
+        let old = termios::tcgetattr(stdin())?;
+        let mut new = old.clone();
+        new.local_modes.remove(termios::LocalModes::ECHO);
+        termios::tcsetattr(stdin(), termios::OptionalActions::Flush, &new)?;
+
+        eprint!("Enter passphrase: ");
+
+        let mut line = Zeroizing::new(String::new());
+        let res = stdin().read_line(&mut line);
+        termios::tcsetattr(stdin(), termios::OptionalActions::Flush, &old)?;
+        eprintln!();
+        res?;
+
+        Ok(Self(CString::new(line.trim_end_matches('\n'))?))
+    }
+
+    // blocks indefinitely if no input is available on stdin
+    pub fn new_from_stdin() -> Result<Self> {
+        info!("Trying to read passphrase from stdin...");
+
+        let mut line = Zeroizing::new(String::new());
+        stdin().read_line(&mut line)?;
+
+        Ok(Self(CString::new(line.trim_end_matches('\n'))?))
+    }
+
+    pub fn new_from_file(passphrase_file: impl AsRef<Path>) -> Result<Self> {
         let passphrase_file = passphrase_file.as_ref();
 
         info!(
-            "Attempting to unlock key for filesystem {} with passphrase from file {}",
-            sb.sb().uuid(),
+            "Attempting to unlock key with passphrase from file {}",
             passphrase_file.display()
         );
 
@@ -181,4 +190,40 @@ impl Passphrase {
 
         Ok(Self(CString::new(passphrase.trim_end_matches('\n'))?))
     }
+
+    fn derive(&self, crypt: &bch_sb_field_crypt) -> bch_key {
+        let crypt_ptr = (crypt as *const bch_sb_field_crypt).cast_mut();
+
+        unsafe { bcachefs::derive_passphrase(crypt_ptr, self.get().as_ptr()) }
+    }
+
+    pub fn check(&self, sb: &bch_sb_handle) -> Result<(bch_key, bch_encrypted_key)> {
+        let bch_key_magic = BCH_KEY_MAGIC.as_bytes().read_u64::<LittleEndian>().unwrap();
+
+        let crypt = sb
+            .sb()
+            .crypt()
+            .ok_or_else(|| anyhow!("filesystem is not encrypted"))?;
+        let mut sb_key = *crypt.key();
+
+        ensure!(
+            sb_key.magic != bch_key_magic,
+            "filesystem does not have encryption key"
+        );
+
+        let mut passphrase_key: bch_key = self.derive(crypt);
+
+        let ret = unsafe {
+            bch2_chacha_encrypt_key(
+                ptr::addr_of_mut!(passphrase_key),
+                sb.sb().nonce(),
+                ptr::addr_of_mut!(sb_key).cast(),
+                mem::size_of_val(&sb_key),
+            )
+        };
+        ensure!(ret == 0, "error encrypting key");
+        ensure!(sb_key.magic == bch_key_magic, "incorrect passphrase");
+
+        Ok((passphrase_key, sb_key))
+    }
 }
